%------
% Project: 2023-code-CDC-A-Distributed-Linear-Quadratic-Discrete-Time-Game-Approach-to-Formation-Control-with-Collision-Avoidance
% Copyright: Institute of Control Systems, Hamburg University of Technology
% License: 
% References:
% Authors: Prima Aditya and Herbert Werner
%------

%---------------------------------------------------------------------------------------------
% For Paper, 
% "A Distributed Linear Quadratic Discrete-Time Game Approach to Formation Control with Collision Avoidance"
% by Prima Aditya and Herbert Werner
% Copyright (c) Institute of Control Systems, Hamburg University of Technology. All rights reserved.
% Licensed under the GPLv3. See LICENSE in the project root for license information.
%--------------------------------------------------------------------------------------------

clc;clear all;close all;

n    = 2;     %dimensional plane
N    = 4;     %number of agents
M    = 5;     %number of edges
m    = n*N;   %adjusting the agents dimension
h    = n*M;   %adjusting the 'edge' agents dimension
tf   = 20;    %simulation time
dt   = 0.1;   %sampling time
T    = tf/dt; %finite time
Hor  = 10;    %prediction horizon
kmax = 12;    %state-dependent Riccati iteration

%initialisation for coupled problem (Problem 1)
F  = [eye(m), zeros(m,1), dt*eye(m);
     zeros(1,m), 1, zeros(1,m);
     zeros(m), zeros(m,1), eye(m)];         %state matrix
G  = [dt^2/2*eye(m); zeros(1,m);dt*eye(m)]; %input matrix
Dn = [1 1 0 0 0;
     -1 0 1 1 0;
      0 -1 -1 0 1;
      0 0 0 -1 -1];
D  = kron(Dn,eye(n));                       %incidence matrix
Phi = D';
d12 = [1.5;1]; d13 = [0;2]; d23 = [-1.5;1]; d24 = [-3;0]; d34 = [-1.5;-1];
d   = [d12;d13;d23;d24;d34];                %formation offset vectore
r   = 0.5;                                  %safety distance
p1  = [3.5;1];    p2 = [12;1];  p3 = [0;5];   p4 = [15;3.5]; %initial positions
v1  = [0.5;1];    v2 = [0;0];   v3 = [0;0];   v4 = [0;0];    %initial velocities
x(:,1) = [p1;p2;p3;p4;1;v1;v2;v3;v4]; %augmented initial state
Wa   = kron(eye(M),eye(n));           %diagonal matrix of edge
La   = D*Wa*D';                       %Laplacian matrix
Qa   = [La, -D*Wa*d, zeros(m);
       (-D*Wa*d)', d'*Wa*d, zeros(1,m);
       zeros(m), zeros(m,1), La]; %state weighting matrix for coupled problem
R   = dt*eye(m);                  %input weighting matrix for coupled problem
tol = 1e-3;                       %tolerance for iteration
ep  = 0.001;                      %epsilon for sigma-norms
eps = 1/ep;

%initialisation for edge system (Problem 2)
A           = [eye(h), dt*eye(h);
              zeros(h), eye(h)];             %state matrix
B           = [dt^2/2*eye(h); dt*eye(h)];    %input matrix
x_real(:,1) = [p1;p2;p3;p4;1;v1;v2;v3;v4];   %augmented initial state
xpinv(:,1)  = x_real(:,1);
xt(:,1)     = [p1;p2;p3;p4;v1;v2;v3;v4];     %initial state
Phit        = kron(kron(eye(n),Dn'),eye(2)); %transformation matrix
z(:,1)      = Phit*xt(:,1) - [d12;d13;d23;d24;d34;zeros(h,1)]; %initial edge state
Qa_tild     = mdiag(eye(h),eye(h));          %state weighting matrix for decoupled problem 
Rt          = dt*eye(h);                     %input weighting matrix for coupled problem
lmax        = 10;                            %iteration for distributed steepest descent
Phia        = kron(Dn',eye(n));
Phiai       = pinv(Phia);
u0          = zeros(m,1);
alpha       = 0.1;                           %learning rate for distributed steepest descent
udata       = [];

% %coupled problem (Problem 1) cannot be implemented in a distributed
% manner - just for the sake of cost comparison

% for t=1:T %time index
%     k = 1;
%     xp(:,1) = x(:,t);       %initialize the predicted state at current state
%     K = zeros(m,2*m+1,Hor); %initialize the state feedback gain
%     while(k <= kmax && (k == 1 || max(abs(squeeze(K - Kol)),[], 'all') > tol))
%         Kol = K;
%         for j = 1:Hor
%             xp(:,j+1) = (F + G*K(:,:,j) )*xp(:,j);
%         end
%         %terminal weights
%         WbT = kron(blkdiag(1/(sig(xp(1:2,Hor+1)-xp(3:4,Hor+1))-r^2), 1/(sig(xp(1:2,Hor+1)-xp(5:6,Hor+1))-r^2),...
%                       1/(sig(xp(3:4,Hor+1)-xp(5:6,Hor+1))-r^2),1/(sig(xp(3:4,Hor+1)-xp(7:8,Hor+1))-r^2),...
%                       1/(sig(xp(5:6,Hor+1)-xp(7:8,Hor+1))-r^2)),eye(n));
%         LbT = D*WbT*D';
%         QbT = [LbT, -D*WbT*d, zeros(m);
%                (-D*WbT*d)', d'*WbT*d, zeros(1,m);
%                zeros(m), zeros(m,1), LbT];
%         QT = dt*(Qa + QbT);
%         P(:,:,Hor+1) = QT;
%     for j=Hor:-1:1
%         Wb(:,:,j) = kron(blkdiag(1/(sig(xp(1:2,j)-xp(3:4,j))-r^2), 1/(sig(xp(1:2,j)-xp(5:6,j))-r^2),...
%                       1/(sig(xp(3:4,j)-xp(5:6,j))-r^2),1/(sig(xp(3:4,j)-xp(7:8,j))-r^2),...
%                       1/(sig(xp(5:6,j)-xp(7:8,j))-r^2)),eye(n));
%         Lb(:,:,j) = D*Wb(:,:,j)*D';
%         Qb(:,:,j) = [Lb(:,:,j), -D*Wb(:,:,j)*d, zeros(m);
%                (-D*Wb(:,:,j)*d)', d'*Wb(:,:,j)*d, zeros(1,m);
%                zeros(m), zeros(m,1), Lb(:,:,j)];
%         Q(:,:,j) = dt*(Qa + Qb(:,:,j));
%         %partial derivatives
%         dQdxp1x(:,:,j) = [                                 -dt*((abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         -dt*((d12(1,:)*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d13(1,:)*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                   -dt*((abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         -dt*((d12(2,:)*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d13(2,:)*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                        (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                 -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d12(1,:)*dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                                                       (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d12(2,:)*dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                        (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d13(1,:)*dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                                                       (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d13(2,:)*dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%          -dt*((abs(xp(1,j) - xp(3,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(1,j) - xp(5,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(1,j) - xp(3,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(1,j) - xp(5,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))), (dt*abs(xp(1,j) - xp(3,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), (dt*abs(xp(1,j) - xp(3,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), (dt*abs(xp(1,j) - xp(5,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(1,j) - xp(5,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0, -dt*((d12(1,:)*abs(xp(1,j) - xp(3,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d12(2,:)*abs(xp(1,j) - xp(3,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d13(1,:)*abs(xp(1,j) - xp(5,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d13(2,:)*abs(xp(1,j) - xp(5,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0, -dt*((abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, -dt*((abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0];
%         dQdxp1y(:,:,j) =[                                   -dt*((abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         -dt*((d12(1,:)*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d13(1,:)*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                   -dt*((abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         -dt*((d12(2,:)*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d13(2,:)*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                 -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d12(1,:)*dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                       (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d12(2,:)*dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d13(1,:)*dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                       (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  (d13(2,:)*dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%          -dt*((abs(xp(2,j) - xp(4,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(2,j) - xp(6,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(2,j) - xp(4,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(2,j) - xp(6,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))), (dt*abs(xp(2,j) - xp(4,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), (dt*abs(xp(2,j) - xp(4,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), (dt*abs(xp(2,j) - xp(6,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(2,j) - xp(6,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0, -dt*((d12(1,:)*abs(xp(2,j) - xp(4,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d12(2,:)*abs(xp(2,j) - xp(4,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d13(1,:)*abs(xp(2,j) - xp(6,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d13(2,:)*abs(xp(2,j) - xp(6,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0, -dt*((abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, -dt*((abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0;
%                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0, 0, 0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, 0, 0];
%         dQdxp2x(:,:,j) =[                 (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d12(1,:)*dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d12(2,:)*dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                          -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                    -dt*((abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -dt*((d12(1,:)*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d23(1,:)*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d24(1,:)*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                 -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                    -dt*((abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -dt*((d12(2,:)*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d24(2,:)*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                 -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d23(1,:)*dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d23(2,:)*dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d24(1,:)*dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d24(2,:)*dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%          (dt*abs(xp(1,j) - xp(3,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), (dt*abs(xp(1,j) - xp(3,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), -dt*((abs(xp(1,j) - xp(3,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(7,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(1,j) - xp(3,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(7,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))), (dt*abs(xp(3,j) - xp(5,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(3,j) - xp(5,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(3,j) - xp(7,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(3,j) - xp(7,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), -dt*((d23(1,:)*abs(xp(3,j) - xp(5,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (d12(2,:)*abs(xp(1,j) - xp(3,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) - (d12(1,:)*abs(xp(1,j) - xp(3,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(3,j) - xp(5,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d24(1,:)*abs(xp(3,j) - xp(7,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d24(2,:)*abs(xp(3,j) - xp(7,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0, -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -dt*((abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, -dt*((abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(1,j) - xp(3,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(3,j) + conj(xp(1,j)) - conj(xp(3,j))))/(((conj(xp(1,j)) - conj(xp(3,j)))*(xp(1,j) - xp(3,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2));
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))];
%         dQdxp2y(:,:,j)=[                  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d12(1,:)*dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d12(2,:)*dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                          -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                    -dt*((abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -dt*((d12(1,:)*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d23(1,:)*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d24(1,:)*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                 -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                    -dt*((abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -dt*((d12(2,:)*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d24(2,:)*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                 -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d23(1,:)*dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d23(2,:)*dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d24(1,:)*dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    (d24(2,:)*dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%          (dt*abs(xp(2,j) - xp(4,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), (dt*abs(xp(2,j) - xp(4,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)), -dt*((abs(xp(2,j) - xp(4,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(8,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(2,j) - xp(4,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(8,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))), (dt*abs(xp(4,j) - xp(6,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(4,j) - xp(6,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(4,j) - xp(8,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(4,j) - xp(8,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), -dt*((d23(1,:)*abs(xp(4,j) - xp(6,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (d12(2,:)*abs(xp(2,j) - xp(4,j))*conj(d12(2,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) - (d12(1,:)*abs(xp(2,j) - xp(4,j))*conj(d12(1,:))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(4,j) - xp(6,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d24(1,:)*abs(xp(4,j) - xp(8,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d24(2,:)*abs(xp(4,j) - xp(8,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0, -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -dt*((abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, -dt*((abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(2,j) - xp(4,j))*(eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(4,j) + conj(xp(2,j)) - conj(xp(4,j))))/(((conj(xp(2,j)) - conj(xp(4,j)))*(xp(2,j) - xp(4,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(3,j))^2/eps + abs(xp(2,j) - xp(4,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2));
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2))];
%         dQdxp3x(:,:,j)=[                  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d13(1,:)*dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d13(2,:)*dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d23(1,:)*dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d23(2,:)*dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                          -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                     dt*((abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -dt*((d13(1,:)*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(1,:)*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d34(1,:)*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                 -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                     dt*((abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -dt*((d13(2,:)*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d34(2,:)*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                 -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d34(1,:)*dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d34(2,:)*dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%          (dt*abs(xp(1,j) - xp(5,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(1,j) - xp(5,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(3,j) - xp(5,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(3,j) - xp(5,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), -dt*((abs(xp(1,j) - xp(5,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(1,j) - xp(5,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), (dt*abs(xp(5,j) - xp(7,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(5,j) - xp(7,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), dt*((d13(1,:)*abs(xp(1,j) - xp(5,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d13(2,:)*abs(xp(1,j) - xp(5,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(1,:)*abs(xp(3,j) - xp(5,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(3,j) - xp(5,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (d34(1,:)*abs(xp(5,j) - xp(7,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)) - (d34(2,:)*abs(xp(5,j) - xp(7,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0, -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, dt*((abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, dt*((abs(xp(1,j) - xp(5,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(1,j) - xp(5,j) + conj(xp(1,j)) - conj(xp(5,j))))/(((conj(xp(1,j)) - conj(xp(5,j)))*(xp(1,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(3,j) - xp(5,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(5,j) + conj(xp(3,j)) - conj(xp(5,j))))/(((conj(xp(3,j)) - conj(xp(5,j)))*(xp(3,j) - xp(5,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2));
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))];
%         dQdxp3y(:,:,j) =[           (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d13(1,:)*dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d13(2,:)*dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d23(1,:)*dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d23(2,:)*dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                          -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                     dt*((abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -dt*((d13(1,:)*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(1,:)*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d34(1,:)*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                 -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                     dt*((abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -dt*((d13(2,:)*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d34(2,:)*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                 -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d34(1,:)*dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   (d34(2,:)*dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%          (dt*abs(xp(2,j) - xp(6,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(2,j) - xp(6,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(4,j) - xp(6,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), (dt*abs(xp(4,j) - xp(6,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)), -dt*((abs(xp(2,j) - xp(6,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(2,j) - xp(6,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), (dt*abs(xp(6,j) - xp(8,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(6,j) - xp(8,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), dt*((d13(1,:)*abs(xp(2,j) - xp(6,j))*conj(d13(1,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d13(2,:)*abs(xp(2,j) - xp(6,j))*conj(d13(2,:))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(1,:)*abs(xp(4,j) - xp(6,j))*conj(d23(1,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) + (d23(2,:)*abs(xp(4,j) - xp(6,j))*conj(d23(2,:))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (d34(1,:)*abs(xp(6,j) - xp(8,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)) - (d34(2,:)*abs(xp(6,j) - xp(8,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0, -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, dt*((abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, dt*((abs(xp(2,j) - xp(6,j))*(eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(2,j) - xp(6,j) + conj(xp(2,j)) - conj(xp(6,j))))/(((conj(xp(2,j)) - conj(xp(6,j)))*(xp(2,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(1,j) - xp(5,j))^2/eps + abs(xp(2,j) - xp(6,j))^2/eps + 1)^(1/2)) + (abs(xp(4,j) - xp(6,j))*(eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(6,j) + conj(xp(4,j)) - conj(xp(6,j))))/(((conj(xp(4,j)) - conj(xp(6,j)))*(xp(4,j) - xp(6,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(5,j))^2/eps + abs(xp(4,j) - xp(6,j))^2/eps + 1)^(1/2)) - (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2));
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0, -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0;
%                                                                                                                                                                                                                                                                                                                                           0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))];
%         dQdxp4x(:,:,j) =[ 0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d24(1,:)*dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d24(2,:)*dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d34(1,:)*dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d34(2,:)*dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                 -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                    dt*((abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -dt*((d24(1,:)*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(1,:)*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                    dt*((abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -dt*((d24(2,:)*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(2,:)*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0, (dt*abs(xp(3,j) - xp(7,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(3,j) - xp(7,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(5,j) - xp(7,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(5,j) - xp(7,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), -dt*((abs(xp(3,j) - xp(7,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(3,j) - xp(7,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), dt*((d24(1,:)*abs(xp(3,j) - xp(7,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d24(2,:)*abs(xp(3,j) - xp(7,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(1,:)*abs(xp(5,j) - xp(7,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(2,:)*abs(xp(5,j) - xp(7,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2));
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2));
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0, -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, dt*((abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0, dt*((abs(xp(3,j) - xp(7,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(3,j) - xp(7,j) + conj(xp(3,j)) - conj(xp(7,j))))/(((conj(xp(3,j)) - conj(xp(7,j)))*(xp(3,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(5,j) - xp(7,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(5,j) - xp(7,j) + conj(xp(5,j)) - conj(xp(7,j))))/(((conj(xp(5,j)) - conj(xp(7,j)))*(xp(5,j) - xp(7,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)))];
%         dQdxp4y(:,:,j) =[ 0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d24(1,:)*dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d24(2,:)*dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d34(1,:)*dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                      -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 (d34(2,:)*dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                 -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                                    dt*((abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -dt*((d24(1,:)*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(1,:)*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                  0,                 -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                    dt*((abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        -dt*((d24(2,:)*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(2,:)*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0, (dt*abs(xp(4,j) - xp(8,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(4,j) - xp(8,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(6,j) - xp(8,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), (dt*abs(xp(6,j) - xp(8,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)), -dt*((abs(xp(4,j) - xp(8,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), -dt*((abs(xp(4,j) - xp(8,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/((conj(r)^2 - (eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2)^2*((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), dt*((d24(1,:)*abs(xp(4,j) - xp(8,j))*conj(d24(1,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d24(2,:)*abs(xp(4,j) - xp(8,j))*conj(d24(2,:))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(1,:)*abs(xp(6,j) - xp(8,j))*conj(d34(1,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)) + (d34(2,:)*abs(xp(6,j) - xp(8,j))*conj(d34(2,:))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))), 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2));
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                  0,  (dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0,                                                                                                                                                                                                                                                                                                                   -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2));
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0, -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, dt*((abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2))),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0;
%          0, 0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                  0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      0, 0, 0,                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                  0, -(dt*abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)),                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    0, dt*((abs(xp(4,j) - xp(8,j))*(eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(4,j) - xp(8,j) + conj(xp(4,j)) - conj(xp(8,j))))/(((conj(xp(4,j)) - conj(xp(8,j)))*(xp(4,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(3,j) - xp(7,j))^2/eps + abs(xp(4,j) - xp(8,j))^2/eps + 1)^(1/2)) + (abs(xp(6,j) - xp(8,j))*(eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)*(xp(6,j) - xp(8,j) + conj(xp(6,j)) - conj(xp(8,j))))/(((conj(xp(6,j)) - conj(xp(8,j)))*(xp(6,j) - xp(8,j)))^(1/2)*((eps*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*(abs(xp(5,j) - xp(7,j))^2/eps + abs(xp(6,j) - xp(8,j))^2/eps + 1)^(1/2)))];
%         %Riccati gain
%         P(:,:,j)  = Q(:,:,j) + [xp(:,j)'*dQdxp1x(:,:,j);xp(:,j)'*dQdxp1y(:,:,j);xp(:,j)'*dQdxp2x(:,:,j);xp(:,j)'*dQdxp2y(:,:,j);...
%                                 xp(:,j)'*dQdxp3x(:,:,j);xp(:,j)'*dQdxp3y(:,:,j);xp(:,j)'*dQdxp4x(:,:,j);xp(:,j)'*dQdxp4y(:,:,j);zeros(m+1,2*m+1)] + F'*P(:,:,j+1)*F - ...
%                                 F'*P(:,:,j+1)*G*inv(R+G'*P(:,:,j+1)*G)*G'*P(:,:,j+1)*F;
%         K(:,:,j)  = -inv(R+G'*P(:,:,j+1)*G)*G'*P(:,:,j+1)*F;
%         end
%         k    = k+1;
%     end
%     u(:,t)   = K(:,:,1)*x(:,t);
%     x(:,t+1) = (F+G*K(:,:,1))*x(:,t);
% end

% %decoupled problem (Problem 2)
for t = 1:T
    %iteration A
    k = 1;
    zp(:,1) = z(:,t);           %initialize the predicted state at current state
    K_tild         = zeros(h,2*h,Hor); %initialize the state feedback gain
    while(k <= kmax && (k == 1 || max(abs(squeeze(K_tild - Kold)),[], 'all') > tol))
        Kold  = K_tild;
        for j = 1:Hor%predict the trajectory
           zp(:,j+1)= (A + B*K_tild(:,:,j))*zp(:,j); 
        end
        %terminal weights
        QbT_tild           = blkdiag(1/(sig(zp(1:2,Hor+1)+d12)^2-r^2), 1/(sig(zp(1:2,Hor+1)+d12)^2-r^2), 1/(sig(zp(3:4,Hor+1)+d13)^2-r^2), 1/(sig(zp(3:4,Hor+1)+d13)^2-r^2),...
                                     1/(sig(zp(5:6,Hor+1)+d23)^2-r^2), 1/(sig(zp(5:6,Hor+1)+d23)^2-r^2), 1/(sig(zp(7:8,Hor+1)+d24)^2-r^2), 1/(sig(zp(7:8,Hor+1)+d24)^2-r^2),1/(sig(zp(9:10,Hor+1)+d34)^2-r^2), 1/(sig(zp(9:10,Hor+1)+d34)^2-r^2),...
                                     1/(sig(zp(1:2,Hor+1)+d12)^2-r^2), 1/(sig(zp(1:2,Hor+1)+d12)^2-r^2), 1/(sig(zp(3:4,Hor+1)+d13)^2-r^2), 1/(sig(zp(3:4,Hor+1)+d13)^2-r^2),...
                                     1/(sig(zp(5:6,Hor+1)+d23)^2-r^2), 1/(sig(zp(5:6,Hor+1)+d23)^2-r^2), 1/(sig(zp(7:8,Hor+1)+d24)^2-r^2), 1/(sig(zp(7:8,Hor+1)+d24)^2-r^2),1/(sig(zp(9:10,Hor+1)+d34)^2-r^2), 1/(sig(zp(9:10,Hor+1)+d34)^2-r^2));
        QT_tild            = dt*( Qa_tild + QbT_tild); 
        P_tild(:,:,Hor+1)  = QT_tild;
        for j = Hor:-1:1
            Qb_tild(:,:,j) = blkdiag(1/(sig(zp(1:2,j)+d12)^2-r^2), 1/(sig(zp(1:2,j)+d12)^2-r^2), 1/(sig(zp(3:4,j)+d13)^2-r^2), 1/(sig(zp(3:4,j)+d13)^2-r^2),...
                                     1/(sig(zp(5:6,j)+d23)^2-r^2), 1/(sig(zp(5:6,j)+d23)^2-r^2), 1/(sig(zp(7:8,j)+d24)^2-r^2), 1/(sig(zp(7:8,j)+d24)^2-r^2),1/(sig(zp(9:10,j)+d34)^2-r^2), 1/(sig(zp(9:10,j)+d34)^2-r^2),...
                                     1/(sig(zp(1:2,j)+d12)^2-r^2), 1/(sig(zp(1:2,j)+d12)^2-r^2), 1/(sig(zp(3:4,j)+d13)^2-r^2), 1/(sig(zp(3:4,j)+d13)^2-r^2),...
                                     1/(sig(zp(5:6,j)+d23)^2-r^2), 1/(sig(zp(5:6,j)+d23)^2-r^2), 1/(sig(zp(7:8,j)+d24)^2-r^2), 1/(sig(zp(7:8,j)+d24)^2-r^2),1/(sig(zp(9:10,j)+d34)^2-r^2), 1/(sig(zp(9:10,j)+d34)^2-r^2));
            Q_tild(:,:,j)  = dt*(Qa_tild + Qb_tild(:,:,j)); 
            %partial derivatives
               dQdq1x(:,:,j) = blkdiag(-(dt*abs(d12(1,:) + zp(1,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(1,:) + zp(1,j) + conj(d12(1,:)) + conj(zp(1,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(1,:) + zp(1,j))*(conj(d12(1,:)) + conj(zp(1,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d12(1,:) + zp(1,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(1,:) + zp(1,j) + conj(d12(1,:)) + conj(zp(1,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(1,:) + zp(1,j))*(conj(d12(1,:)) + conj(zp(1,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        0, 0, 0, 0, 0, 0, 0, 0,...
                                        -(dt*abs(d12(1,:) + zp(1,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(1,:) + zp(1,j) + conj(d12(1,:)) + conj(zp(1,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(1,:) + zp(1,j))*(conj(d12(1,:)) + conj(zp(1,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d12(1,:) + zp(1,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(1,:) + zp(1,j) + conj(d12(1,:)) + conj(zp(1,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(1,:) + zp(1,j))*(conj(d12(1,:)) + conj(zp(1,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        0, 0, 0, 0, 0, 0, 0, 0);
               
               dQdq1y(:,:,j) = blkdiag(-(dt*abs(d12(2,:) + zp(2 ,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(2,:) + zp(2 ,j) + conj(d12(2,:)) + conj(zp(2 ,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(2,:) + zp(2 ,j))*(conj(d12(2,:)) + conj(zp(2 ,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d12(2,:) + zp(2 ,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(2,:) + zp(2 ,j) + conj(d12(2,:)) + conj(zp(2 ,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(2,:) + zp(2 ,j))*(conj(d12(2,:)) + conj(zp(2 ,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        0,0,0,0,0,0,0,0,...
                                        -(dt*abs(d12(2,:) + zp(2 ,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(2,:) + zp(2 ,j) + conj(d12(2,:)) + conj(zp(2 ,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(2,:) + zp(2 ,j))*(conj(d12(2,:)) + conj(zp(2 ,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d12(2,:) + zp(2 ,j))*(eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)*(d12(2,:) + zp(2 ,j) + conj(d12(2,:)) + conj(zp(2 ,j))))/(((eps*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d12(2,:) + zp(2 ,j))*(conj(d12(2,:)) + conj(zp(2 ,j))))^(1/2)*(abs(d12(1,:) + zp(1,j))^2/eps + abs(d12(2,:) + zp(2 ,j))^2/eps + 1)^(1/2)),...
                                        0,0,0,0,0,0,0,0);
               
               dQdq2x(:,:,j) = blkdiag(0, 0,...
                                        -(dt*abs(d13(1,:) + zp(3 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(1,:) + zp(3 ,j) + conj(d13(1,:)) + conj(zp(3 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(1,:) + zp(3 ,j))*(conj(d13(1,:)) + conj(zp(3 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d13(1,:) + zp(3 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(1,:) + zp(3 ,j) + conj(d13(1,:)) + conj(zp(3 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(1,:) + zp(3 ,j))*(conj(d13(1,:)) + conj(zp(3 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        0, 0, 0, 0, 0, 0, 0, 0,...
                                        -(dt*abs(d13(1,:) + zp(3 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(1,:) + zp(3 ,j) + conj(d13(1,:)) + conj(zp(3 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(1,:) + zp(3 ,j))*(conj(d13(1,:)) + conj(zp(3 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d13(1,:) + zp(3 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(1,:) + zp(3 ,j) + conj(d13(1,:)) + conj(zp(3 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(1,:) + zp(3 ,j))*(conj(d13(1,:)) + conj(zp(3 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        0,0,0,0,0,0);  

               dQdq2y(:,:,j) =blkdiag(0,0,...
                                        -(dt*abs(d13(2,:) + zp(4 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(2,:) + zp(4 ,j) + conj(d13(2,:)) + conj(zp(4 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(2,:) + zp(4 ,j))*(conj(d13(2,:)) + conj(zp(4 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d13(2,:) + zp(4 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(2,:) + zp(4 ,j) + conj(d13(2,:)) + conj(zp(4 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(2,:) + zp(4 ,j))*(conj(d13(2,:)) + conj(zp(4 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        0, 0, 0, 0, 0, 0, 0, 0,...
                                        -(dt*abs(d13(2,:) + zp(4 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(2,:) + zp(4 ,j) + conj(d13(2,:)) + conj(zp(4 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(2,:) + zp(4 ,j))*(conj(d13(2,:)) + conj(zp(4 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d13(2,:) + zp(4 ,j))*(eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)*(d13(2,:) + zp(4 ,j) + conj(d13(2,:)) + conj(zp(4 ,j))))/(((eps*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d13(2,:) + zp(4 ,j))*(conj(d13(2,:)) + conj(zp(4 ,j))))^(1/2)*(abs(d13(1,:) + zp(3 ,j))^2/eps + abs(d13(2,:) + zp(4 ,j))^2/eps + 1)^(1/2)),...
                                        0,0,0,0,0,0);

               dQdq3x(:,:,j)= blkdiag(0, 0, 0, 0,...
                                        -(dt*abs(d23(1,:) + zp(5 ,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(1,:) + zp(5 ,j) + conj(d23(1,:)) + conj(zp(5 ,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(1,:) + zp(5 ,j))*(conj(d23(1,:)) + conj(zp(5 ,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d23(1,:) + zp(5 ,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(1,:) + zp(5 ,j) + conj(d23(1,:)) + conj(zp(5 ,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(1,:) + zp(5 ,j))*(conj(d23(1,:)) + conj(zp(5 ,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                        0, 0, 0, 0, 0, 0, 0, 0,...
                                        -(dt*abs(d23(1,:) + zp(5 ,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(1,:) + zp(5 ,j) + conj(d23(1,:)) + conj(zp(5 ,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(1,:) + zp(5 ,j))*(conj(d23(1,:)) + conj(zp(5 ,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                        -(dt*abs(d23(1,:) + zp(5 ,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(1,:) + zp(5 ,j) + conj(d23(1,:)) + conj(zp(5 ,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(1,:) + zp(5 ,j))*(conj(d23(1,:)) + conj(zp(5 ,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                        0,0,0,0);

               dQdq3y(:,:,j)=  blkdiag(0, 0, 0, 0,...
                                         -(dt*abs(d23(2,:) + zp(6,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(2,:) + zp(6,j) + conj(d23(2,:)) + conj(zp(6,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(2,:) + zp(6,j))*(conj(d23(2,:)) + conj(zp(6,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d23(2,:) + zp(6,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(2,:) + zp(6,j) + conj(d23(2,:)) + conj(zp(6,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(2,:) + zp(6,j))*(conj(d23(2,:)) + conj(zp(6,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                         0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d23(2,:) + zp(6,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(2,:) + zp(6,j) + conj(d23(2,:)) + conj(zp(6,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(2,:) + zp(6,j))*(conj(d23(2,:)) + conj(zp(6,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d23(2,:) + zp(6,j))*(eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)*(d23(2,:) + zp(6,j) + conj(d23(2,:)) + conj(zp(6,j))))/(((eps*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d23(2,:) + zp(6,j))*(conj(d23(2,:)) + conj(zp(6,j))))^(1/2)*(abs(d23(1,:) + zp(5 ,j))^2/eps + abs(d23(2,:) + zp(6,j))^2/eps + 1)^(1/2)),...
                                         0,0,0,0);
               
               dQdq4x(:,:,j) = blkdiag(0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d24(1,:) + zp(7,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(1,:) + zp(7,j) + conj(d24(1,:)) + conj(zp(7,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(1,:) + zp(7,j))*(conj(d24(1,:)) + conj(zp(7,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d24(1,:) + zp(7,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(1,:) + zp(7,j) + conj(d24(1,:)) + conj(zp(7,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(1,:) + zp(7,j))*(conj(d24(1,:)) + conj(zp(7,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d24(1,:) + zp(7,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(1,:) + zp(7,j) + conj(d24(1,:)) + conj(zp(7,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(1,:) + zp(7,j))*(conj(d24(1,:)) + conj(zp(7,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d24(1,:) + zp(7,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(1,:) + zp(7,j) + conj(d24(1,:)) + conj(zp(7,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(1,:) + zp(7,j))*(conj(d24(1,:)) + conj(zp(7,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         0,0);
               
               dQdq4y(:,:,j) = blkdiag(0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d24(2,:) + zp(8,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(2,:) + zp(8,j) + conj(d24(2,:)) + conj(zp(8,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(2,:) + zp(8,j))*(conj(d24(2,:)) + conj(zp(8,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d24(2,:) + zp(8,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(2,:) + zp(8,j) + conj(d24(2,:)) + conj(zp(8,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(2,:) + zp(8,j))*(conj(d24(2,:)) + conj(zp(8,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d24(2,:) + zp(8,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(2,:) + zp(8,j) + conj(d24(2,:)) + conj(zp(8,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(2,:) + zp(8,j))*(conj(d24(2,:)) + conj(zp(8,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d24(2,:) + zp(8,j))*(eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)*(d24(2,:) + zp(8,j) + conj(d24(2,:)) + conj(zp(8,j))))/(((eps*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d24(2,:) + zp(8,j))*(conj(d24(2,:)) + conj(zp(8,j))))^(1/2)*(abs(d24(1,:) + zp(7,j))^2/eps + abs(d24(2,:) + zp(8,j))^2/eps + 1)^(1/2)),...
                                         0,0); 
               
               dQdq5x(:,:,j) = blkdiag(0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d34(1,:) + zp(9,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(1,:) + zp(9,j) + conj(d34(1,:)) + conj(zp(9,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(1,:) + zp(9,j))*(conj(d34(1,:)) + conj(zp(9,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d34(1,:) + zp(9,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(1,:) + zp(9,j) + conj(d34(1,:)) + conj(zp(9,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(1,:) + zp(9,j))*(conj(d34(1,:)) + conj(zp(9,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)),...
                                         0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d34(1,:) + zp(9,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(1,:) + zp(9,j) + conj(d34(1,:)) + conj(zp(9,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(1,:) + zp(9,j))*(conj(d34(1,:)) + conj(zp(9,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d34(1,:) + zp(9,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(1,:) + zp(9,j) + conj(d34(1,:)) + conj(zp(9,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(1,:) + zp(9,j))*(conj(d34(1,:)) + conj(zp(9,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)));
     
               dQdq5y(:,:,j) = blkdiag(0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d34(2,:) + zp(10,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(2,:) + zp(10,j) + conj(d34(2,:)) + conj(zp(10,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(2,:) + zp(10,j))*(conj(d34(2,:)) + conj(zp(10,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d34(2,:) + zp(10,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(2,:) + zp(10,j) + conj(d34(2,:)) + conj(zp(10,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(2,:) + zp(10,j))*(conj(d34(2,:)) + conj(zp(10,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)),...
                                         0, 0, 0, 0, 0, 0, 0, 0,...
                                         -(dt*abs(d34(2,:) + zp(10,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(2,:) + zp(10,j) + conj(d34(2,:)) + conj(zp(10,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(2,:) + zp(10,j))*(conj(d34(2,:)) + conj(zp(10,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)),...
                                         -(dt*abs(d34(2,:) + zp(10,j))*(eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)*(d34(2,:) + zp(10,j) + conj(d34(2,:)) + conj(zp(10,j))))/(((eps*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2) - eps)^2 - r^2)^2*((d34(2,:) + zp(10,j))*(conj(d34(2,:)) + conj(zp(10,j))))^(1/2)*(abs(d34(1,:) + zp(9,j))^2/eps + abs(d34(2,:) + zp(10,j))^2/eps + 1)^(1/2)));
              
               %Riccati gain
            P_tild(:,:,j)  = Q_tild(:,:,j) + [zp(:,j)'*dQdq1x(:,:,j);zp(:,j)'*dQdq1y(:,:,j);zp(:,j)'*dQdq2x(:,:,j);zp(:,j)'*dQdq2y(:,:,j);zp(:,j)'*dQdq3x(:,:,j);zp(:,j)'*dQdq3y(:,:,j);zp(:,j)'*dQdq4x(:,:,j);zp(:,j)'*dQdq4y(:,:,j);zp(:,j)'*dQdq5x(:,:,j);zp(:,j)'*dQdq5y(:,:,j);zeros(2*M,2*h)] + A'*P_tild(:,:,j+1)*A - ...
                              A'*P_tild(:,:,j+1)*B*inv(Rt+B'*P_tild(:,:,j+1)*B)*B'*P_tild(:,:,j+1)*A;
            K_tild(:,:,j)  = -inv(Rt+B'*P_tild(:,:,j+1)*B)*B'*P_tild(:,:,j+1)*A;
        end
        k    = k+1;
    end 
    a(:,t)   = K_tild(:,:,1)*z(:,t);
    z(:,t+1) = A*z(:,t) + B*K_tild(:,:,1)*z(:,t); 
    % centralised solution
    for v = 1:N
         upinv(:,t) = Phiai * a(:,t);
    end
    xpinv(:,t+1) = F * xpinv(:,t) + G*upinv(:,t); 
    % iteration B
    l = 1;
    if t > 1
        u0 = usol;
    end
    while (l <= lmax)
        u0_temp = (eye(m) - 2*alpha*Phia'*Phia)*u0(:,l) + 2*alpha*Phia'*a(:,t); 
        u0 = [u0 u0_temp];
    l = l+1;
    end
    udata         = [udata,u0];
    %toc;
    usol          = u0(:,end);
    uhat(:,t)     = usol;   
    x_real(:,t+1) = F * x_real(:,t) + G * uhat(:,t);
end

% zNash = [x(1:2,:)-x(3:4,:)-d12; x(1:2,:)-x(5:6,:)-d13; x(3:4,:)-x(5:6,:)-d23;...
%          x(3:4,:)-x(7:8,:)-d24; x(5:6,:)-x(7:8,:)-d34; x(10:11,:)-x(12:13,:);...
%          x(10:11,:)-x(14:15,:); x(12:13,:)-x(14:15,:); x(12:13,:)-x(16:17,:); x(14:15,:)-x(16:17,:)];
% aNash = Phi*u;
% 
% %calculate the cost
% for k = 1:T
%     %individual edge agents
%     J1Nash_(k) = [zNash(1:2,k);zNash(11:12,k)]'*blkdiag(Q_tild(1,1,1),Q_tild(2,2,1),Q_tild(11,11,1),Q_tild(12,12,1))*[zNash(1:2,k);zNash(11:12,k)] + aNash(1:2,k)'*Rt(1:2,1:2)*aNash(1:2,k);
%     J2Nash_(k) = [zNash(3:4,k);zNash(13:14,k)]'*blkdiag(Q_tild(3,3,1),Q_tild(4,4,1),Q_tild(13,13,1),Q_tild(14,14,1))*[zNash(3:4,k);zNash(13:14,k)] + aNash(3:4,k)'*Rt(3:4,3:4)*aNash(3:4,k);
%     J3Nash_(k) = [zNash(5:6,k);zNash(15:16,k)]'*blkdiag(Q_tild(5,5,1),Q_tild(6,6,1),Q_tild(15,15,1),Q_tild(16,16,1))*[zNash(5:6,k);zNash(15:16,k)] + aNash(5:6,k)'*Rt(5:6,5:6)*aNash(5:6,k);
%     J4Nash_(k) = [zNash(7:8,k);zNash(17:18,k)]'*blkdiag(Q_tild(7,7,1),Q_tild(8,8,1),Q_tild(17,17,1),Q_tild(18,18,1))*[zNash(7:8,k);zNash(17:18,k)] + aNash(7:8,k)'*Rt(7:8,7:8)*aNash(7:8,k);
%     J5Nash_(k) = [zNash(9:10,k);zNash(19:20,k)]'*blkdiag(Q_tild(9,9,1),Q_tild(10,10,1),Q_tild(19,19,1),Q_tild(20,20,1))*[zNash(9:10,k);zNash(19:20,k)] + aNash(9:10,k)'*Rt(9:10,9:10)*aNash(9:10,k);
%     J1Par_(k)  = [z(1:2,k);z(11:12,k)]'*blkdiag(Q_tild(1,1,1),Q_tild(2,2,1),Q_tild(11,11),Q_tild(12,12,1))*[z(1:2,k);z(11:12,k)] + a(1:2,k)'*Rt(1:2,1:2)*a(1:2,k);
%     J2Par_(k)  = [z(3:4,k);z(13:14,k)]'*blkdiag(Q_tild(3,3,1),Q_tild(4,4,1),Q_tild(13,13),Q_tild(14,14,1))*[z(3:4,k);z(13:14,k)] + a(3:4,k)'*Rt(3:4,3:4)*a(3:4,k);
%     J3Par_(k)  = [z(5:6,k);z(15:16,k)]'*blkdiag(Q_tild(5,5,1),Q_tild(6,6,1),Q_tild(15,15),Q_tild(16,16,1))*[z(5:6,k);z(15:16,k)] + a(5:6,k)'*Rt(5:6,5:6)*a(5:6,k);
%     J4Par_(k)  = [z(7:8,k);z(17:18,k)]'*blkdiag(Q_tild(7,7,1),Q_tild(8,8,1),Q_tild(17,17),Q_tild(18,18,1))*[z(7:8,k);z(17:18,k)] + a(7:8,k)'*Rt(7:8,7:8)*a(7:8,k);
%     J5Par_(k)  = [z(9:10,k);z(19:20,k)]'*blkdiag(Q_tild(9,9,1),Q_tild(10,10,1),Q_tild(19,19),Q_tild(20,20,1))*[z(9:10,k);z(19:20,k)] + a(9:10,k)'*Rt(9:10,9:10)*a(9:10,k);
%     %total
%     JNash_(k)  = zNash(:,k)'*Q_tild(:,:,1)*zNash(:,k) + aNash(:,k)'*Rt*aNash(:,k);
%     JPar_(k)   = z(:,k)'*Q_tild(:,:,1)*z(:,k) + a(:,k)'*Rt*a(:,k);
% end
% J1Nash = sum(J1Nash_);J2Nash = sum(J2Nash_);J3Nash = sum(J3Nash_);J4Nash = sum(J4Nash_);J5Nash = sum(J5Nash_);
% J1Par  = sum(J1Par_);J2Par = sum(J2Par_);J3Par = sum(J3Par_);J4Par = sum(J4Par_);J5Par = sum(J5Par_);
% 
% JNash  = sum(JNash_) + zNash(:,end)'*Q_tild(:,:,1)*zNash(:,end); %global cost of coupled problem
% JPar   = sum(JPar_)  + z(:,end)'*Q_tild(:,:,1)*z(:,end); %global cost of decoupled problem

% figure(1)%formation of coupled problem
% plot( x(1,:), x(2,:), x(3,:), x(4,:), x(5,:), x(6,:), x(7,:), x(8,:),'linewidth',3)
% hold all
% p1=plot( x(1,end), x(2,end),'o','linewidth',2.5);
% p1.Color = "#0072BD";
% p2=plot( x(3,end), x(4,end),'o','linewidth',2.5);
% p2.Color = "#D95319";
% p3=plot( x(5,end), x(6,end),'o','linewidth',2.5);
% p3.Color = 	"#EDB120";
% s4=plot( x(7,end), x(8,end),'o','linewidth',2.5);
% s4.Color = 	"#7E2F8E";
% circ1 = viscircles([ x(1,end)  x(2,end)],r,'Color',"#0072BD",'linestyle',':');
% circ2 = viscircles([ x(3,end)  x(4,end)],r,'Color',"#D95319",'linestyle',':');
% circ3 = viscircles([ x(5,end)  x(6,end)],r,'Color',"#EDB120",'linestyle',':');
% circ4 = viscircles([ x(7,end)  x(8,end)],r,'Color',"#7E2F8E",'linestyle',':');
% legend('Agent 1','Agent 2','Agent 3','Agent 4','location','best','fontsize',14)
% xlabel('x-axis','fontsize',12)
% ylabel('y-axis','fontsize',12)
% xlim([-1 16]);
% ylim([0 13]);
% grid on

% xall = xlsread('xall.xlsx','5edges','B2:GT18');
% figure(2)%formation of decoupled problem
% plot(x_real(1,:),x_real(2,:),x_real(3,:),x_real(4,:),x_real(5,:),x_real(6,:),x_real(7,:),x_real(8,:),'linewidth',3)
% hold all
% s1=plot(xall(1,:),xall(2,:),':','linewidth',2.5);
% s1.Color = "#0072BD";
% s2=plot(xall(3,:),xall(4,:),':','linewidth',2.5);
% s2.Color = "#D95319";
% s3=plot(xall(5,:),xall(6,:),':','linewidth',2.5);
% s3.Color = 	"#EDB120";
% s4=plot(xall(7,:),xall(8,:),':','linewidth',2.5);
% s4.Color = 	"#7E2F8E";
% p1=plot(x_real(1,end),x_real(2,end),'o','linewidth',2.5);
% p1.Color = "#0072BD";
% p2=plot(x_real(3,end),x_real(4,end),'o','linewidth',2.5);
% p2.Color = "#D95319";
% p3=plot(x_real(5,end),x_real(6,end),'o','linewidth',2.5);
% p3.Color = 	"#EDB120";
% s4=plot(x_real(7,end),x_real(8,end),'o','linewidth',2.5);
% s4.Color = 	"#7E2F8E";
% circ1 = viscircles([x_real(1,end) x_real(2,end)],r,'Color',"#0072BD",'linestyle',':');
% circ2 = viscircles([x_real(3,end) x_real(4,end)],r,'Color',"#D95319",'linestyle',':');
% circ3 = viscircles([x_real(5,end) x_real(6,end)],r,'Color',"#EDB120",'linestyle',':');
% circ4 = viscircles([x_real(7,end) x_real(8,end)],r,'Color',"#7E2F8E",'linestyle',':');
% legend('Agent 1','Agent 2','Agent 3','Agent 4','location','northwest','fontsize',14)
% xlabel('x-axis','fontsize',12)
% ylabel('y-axis','fontsize',12)
% xlim([-1 16]);
% ylim([0 13]);
% grid on

% To plot the control input of Agent 1 for iteration Iter = 10;
fig = figure(97);clf;
ax  = axes;
plot(ax,udata(1,:),'*','Color',"#0072BD")
hold on
for i = 1:length(upinv(1,:))
    plot(ax,(lmax+1)*i,upinv(1,i),'x','Color',"#D95319",'linewidth',5);
end
grid on
xlabel('Time/ iterations')
ylabel('Control input of Agent 1')
legend('Distributed approach','Centralized solution','fontsize',12,'location','southeast');
xlim([0 750])
options.axes.Names = {'Position','XLim'};
options.axes.Values = {[.42 .42 .45 .45],[100,200]};
options.rectangle.Names = {};
options.rectangle.Values = {};
options.arrows.Names = {'HeadLength','HeadWidth'};
options.arrows.Values = {8,8};
[zoom_utils] = zoom_plot(ax,options);

% To plot the control input of  Agent 3 for iteration Iter = 10;
fig = figure(96);clf;
ax  = axes;
plot(ax,udata(5,:),'*','Color',"#0072BD")
hold on
for i = 1:length(upinv(5,:))
    plot(ax,(lmax+1)*i,upinv(5,i),'x','Color',"#D95319",'linewidth',5);
end
grid on
xlabel('Time/ iterations')
ylabel('Control input of Agent 3')
legend('Distributed approach','Centralized solution','fontsize',12,'location','northeast');
xlim([0 750])
options.axes.Names = {'Position','XLim'};
options.axes.Values = {[.42 .2 .45 .45],[100,200]};
options.rectangle.Names = {};
options.rectangle.Values = {};
options.arrows.Names = {'HeadLength','HeadWidth'};
options.arrows.Values = {8,8};
[zoom_utils] = zoom_plot(ax,options);


% %to plot an animation of coupled problem (Problem 1)
% figure(99)
% set(gcf,'color','w');
% filename='4agents';
% counter=1;
% vid = VideoWriter('4agents.avi','Motion JPEG AVI');
% open(vid)
% for i=1:tf/dt
%     plot(x(1,i),x(2,i),'o',x(3,i),x(4,i),'o',x(5,i),x(6,i),'o',x(7,i),x(8,i),'o','linewidth',4);
%     circ1 = viscircles([x(1,i) x(2,i)],r,'Color',"#0072BD",'linestyle',':');
%     circ2 = viscircles([x(3,i) x(4,i)],r,'Color',"#D95319",'linestyle',':');
%     circ3 = viscircles([x(5,i) x(6,i)],r,'Color',"#EDB120",'linestyle',':');
%     circ4 = viscircles([x(7,i) x(8,i)],r,'Color',"#7E2F8E",'linestyle',':');
%     hold on
%     grid on
%     p1 = plot(x(1,1:i),x(2,1:i),':','linewidth',2);
%     p1.Color = "#0072BD";
%     p2 = plot(x(3,1:i),x(4,1:i),':','linewidth',2);
%     p2.Color = "#D95319";
%     p3 = plot(x(5,1:i),x(6,1:i),':','linewidth',2);
%     p3.Color = 	"#EDB120";
%     p3 = plot(x(7,1:i),x(8,1:i),':','linewidth',2);
%     p3.Color = 	"#7E2F8E";
%     legend('Agent 1','Agent 2','Agent 3','Agent 4','fontsize',14,'location','northwest')
%     xlabel('x-axis','fontsize',14);
%     ylabel('y-axis','fontsize',14);
%     xlim([-1 16]);
%     ylim([0 13]);
%     frame = getframe(99);
%     writeVideo(vid,frame);
% %     im = frame2im(frame);
% %     [imind,cm] = rgb2ind(im,256);
% %     imwrite(imind,cm,strcat(filename,num2str(counter),'.png'));
% %     counter=counter+1;
%     pause(0.01)   
%     if i~= length(1:(tf/dt))
%     clf
%     end
% end
% % close(vid)
% % print -depsc myfig.png

%to plot an animation of decoupled problem (Problem 2)
figure(100)
set(gcf,'color','w');
filename='4agents';
counter=1;
vid = VideoWriter('4agents.avi','Motion JPEG AVI');
open(vid)
for i=1:tf/dt
    plot(x_real(1,i),x_real(2,i),'o',x_real(3,i),x_real(4,i),'o',x_real(5,i),x_real(6,i),'o',x_real(7,i),x_real(8,i),'o','linewidth',4);
    circ1 = viscircles([x_real(1,i) x_real(2,i)],r,'Color',"#0072BD",'linestyle',':');
    circ2 = viscircles([x_real(3,i) x_real(4,i)],r,'Color',"#D95319",'linestyle',':');
    circ3 = viscircles([x_real(5,i) x_real(6,i)],r,'Color',"#EDB120",'linestyle',':');
    circ4 = viscircles([x_real(7,i) x_real(8,i)],r,'Color',"#7E2F8E",'linestyle',':');
    hold on
    grid on
    p1 = plot(x_real(1,1:i),x_real(2,1:i),':','linewidth',2);
    p1.Color = "#0072BD";
    p2 = plot(x_real(3,1:i),x_real(4,1:i),':','linewidth',2);
    p2.Color = "#D95319";
    p3 = plot(x_real(5,1:i),x_real(6,1:i),':','linewidth',2);
    p3.Color = 	"#EDB120";
    p3 = plot(x_real(7,1:i),x_real(8,1:i),':','linewidth',2);
    p3.Color = 	"#7E2F8E";
    legend('Agent 1','Agent 2','Agent 3','Agent 4','fontsize',14,'location','northwest')
    xlabel('x-axis','fontsize',14);
    ylabel('y-axis','fontsize',14);
    xlim([-1 16]);
    ylim([0 13]);
    frame = getframe(100);
    writeVideo(vid,frame);
%     im = frame2im(frame);
%     [imind,cm] = rgb2ind(im,256);
%     imwrite(imind,cm,strcat(filename,num2str(counter),'.png'));
%     counter=counter+1;
    pause(0.01)   
    if i~= length(1:(tf/dt))
    clf
    end
end
close(vid)











